@@ -3,7 +3,7 @@ package TestApp::Controller::Action::Forward;
 use strict;
 use base 'TestApp::Controller::Action';
 
-sub one : Relative {
+sub one : Local {
     my ( $self, $c ) = @_;
     $c->forward('two');
 }
@@ -13,7 +13,7 @@ sub two : Private {
     $c->forward('three');
 }
 
-sub three : Relative {
+sub three : Local {
     my ( $self, $c ) = @_;
     $c->forward('four');
 }
@@ -23,29 +23,46 @@ sub four : Private {
     $c->forward('/action/forward/five');
 }
 
-sub five : Relative {
+sub five : Local {
     my ( $self, $c ) = @_;
     $c->forward('TestApp::View::Dump::Request');
 }
 
 
-sub jojo : Relative {
+sub jojo : Local {
     my ( $self, $c ) = @_;
     $c->forward('one');
     $c->forward('three');
 }
 
 
-sub inheritance : Relative {
+sub inheritance : Local {
     my ( $self, $c ) = @_;
     $c->forward('/action/inheritance/a/b/default');
     $c->forward('five');
 }
 
-sub global : Relative {
+sub global : Local {
     my ( $self, $c ) = @_;
-    $c->forward('/global_action');
+    $c->forward( '/global_action' );
 }
 
+sub with_args : Local {
+    my ( $self, $c, $orig ) = @_;
+    $c->forward( 'args',[qq/new/] );
+    $c->res->body( $c->req->args->[0] );
+}
+
+sub with_method_and_args : Local {
+    my ( $self, $c, $orig ) = @_;
+    $c->forward( qw/TestApp::Controller::Action::Forward args/, [qq/new/] );
+    $c->res->body( $c->req->args->[0] );
+}
+
+sub args : Local {
+    my ( $self, $c, $val ) = @_;
+    die "Expected argument 'new', got '$val'" unless $val eq 'new';
+    die "passed argument does not match args" unless $val eq $c->req->args->[0];
+}
 
 1;
